@@ -33,7 +33,6 @@
 
 class QAction;
 class QmlAppletScript;
-class QSignalMapper;
 class QSizeF;
 
 namespace KDeclarative
@@ -195,7 +194,7 @@ class AppletInterface : public PlasmaQuick::AppletQuickItem
      * Provides access to the geometry of the applet is in.
      * Can be useful to figure out what's the absolute position of the applet.
      */
-    Q_PROPERTY(QRect screenGeometry READ screenGeometry NOTIFY screenChanged)
+    Q_PROPERTY(QRect screenGeometry READ screenGeometry NOTIFY screenGeometryChanged)
 
     /**
      * Whether the dialog should be hidden when the dialog loses focus.
@@ -227,9 +226,21 @@ class AppletInterface : public PlasmaQuick::AppletQuickItem
      */
     Q_PROPERTY(QString configurationRequiredReason READ configurationRequiredReason WRITE setConfigurationRequiredReason NOTIFY configurationRequiredReasonChanged)
 
+    /**
+     * screen area free of panels: the coordinates are relative to the containment,
+     * it's independent from the screen position
+     * For more precise available geometry use availableScreenRegion()
+     */
+    Q_PROPERTY(QRect availableScreenRect READ availableScreenRect NOTIFY availableScreenRectChanged)
+
+    /**
+     * The available region of this screen, panels excluded. It's a list of rectangles
+     */
+    Q_PROPERTY(QVariantList availableScreenRegion READ availableScreenRegion NOTIFY availableScreenRegionChanged)
+
 public:
-    AppletInterface(DeclarativeAppletScript *script, const QVariantList &args = QVariantList(), QQuickItem *parent = 0);
-    ~AppletInterface();
+    AppletInterface(DeclarativeAppletScript *script, const QVariantList &args = QVariantList(), QQuickItem *parent = nullptr);
+    ~AppletInterface() override;
 
 //API not intended for the QML part
 
@@ -266,6 +277,14 @@ public:
 
     Q_INVOKABLE QAction *action(QString name) const;
 
+    /**
+     * Should be called before retrieving any action
+     * to ensure contents are up to date
+     * @see contextualActionsAboutToShow
+     * @since 5.58
+     */
+    Q_INVOKABLE void prepareContextualActions();
+
     /**
      * FIXME: remove?
      * Retrieve the path of a file from the Plasmoid package
@@ -301,6 +320,10 @@ public:
      */
     Q_INVOKABLE QStringList downloadedFiles() const;
 
+    QVariantList availableScreenRegion() const;
+
+    QRect availableScreenRect() const;
+
     static AppletInterface *qmlAttachedProperties(QObject *object)
     {
         return qobject_cast<AppletInterface *>(AppletQuickItem::qmlAttachedProperties(object));
@@ -413,9 +436,12 @@ Q_SIGNALS:
     void backgroundHintsChanged();
     void busyChanged();
     void screenChanged();
+    void screenGeometryChanged();
     void hideOnWindowDeactivateChanged();
     void associatedApplicationChanged();
     void associatedApplicationUrlsChanged();
+    void availableScreenRegionChanged();
+    void availableScreenRectChanged();
 
     void userConfiguringChanged();
     void globalShortcutChanged();
@@ -423,11 +449,11 @@ Q_SIGNALS:
     void configurationRequiredReasonChanged();
 
 protected Q_SLOTS:
-    void init() Q_DECL_OVERRIDE;
+    void init() override;
 
 protected:
-    bool event(QEvent *event) Q_DECL_OVERRIDE;
-    bool eventFilter(QObject *watched, QEvent *event) Q_DECL_OVERRIDE;
+    bool event(QEvent *event) override;
+    bool eventFilter(QObject *watched, QEvent *event) override;
 
 private Q_SLOTS:
     void destroyedChanged(bool destroyed);
@@ -435,7 +461,6 @@ private Q_SLOTS:
 private:
 
     QStringList m_actions;
-    QSignalMapper *m_actionSignals;
 
     KDeclarative::ConfigPropertyMap *m_configuration;
     DeclarativeAppletScript *m_appletScriptEngine;
